#!/usr/bin/python2
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

import gi
import sys

from gi.repository import Gio, GLib
from collections import defaultdict
import json

# Filter names from arguments.
if sys.argv[1:]:
    prefixes = sys.argv[1:]
    activateable = True
else:
    prefixes = ['']
    activateable = False


bus = Gio.bus_get_sync(Gio.BusType.SYSTEM)

names = bus.call_sync("org.freedesktop.DBus",
                      "/",
                      "org.freedesktop.DBus",
                      "ListNames",
                      None, GLib.VariantType('(as)'),
                      Gio.DBusCallFlags.NONE,
                      -1,
                      None)[0]

if activateable:
    activatable_names = bus.call_sync("org.freedesktop.DBus",
                                      "/",
                                      "org.freedesktop.DBus",
                                      "ListActivatableNames",
                                      None, GLib.VariantType('(as)'),
                                      Gio.DBusCallFlags.NONE,
                                      -1,
                                      None)[0]

    names.extend(activatable_names)

# Unique
names = set(names)

jsond = defaultdict(lambda : defaultdict(lambda : dict()))


def node_introspect(name, obj):
    try:
        res = bus.call_sync(name, obj, 'org.freedesktop.DBus.Introspectable', 'Introspect',
                            None, GLib.VariantType.new("(s)"),
                            Gio.DBusCallFlags.NONE,
                            -1,
                            None)[0]
        node_info = Gio.DBusNodeInfo.new_for_xml(res)

        jsond[name][obj]['node-info'] = res

        for node in node_info.nodes:
            path = obj
            if path != '/':
                path += '/'
            path += node.path

            node_introspect(name, path)
            #print(res)
            #node_introspect(name, node)

        if 'interfaces' not in jsond[name][obj]:
            jsond[name][obj]['interfaces'] = {}


        for interface in node_info.interfaces:

            jsond[name][obj]['interfaces'][interface.name] = { 'props' : {} }

            # This is a bit brute force, simply query all interfaces
            if interface.name.startswith("org.freedesktop.DBus"):
                continue

            try:
                res = bus.call_sync(name, obj, 'org.freedesktop.DBus.Properties', 'GetAll',
                                    GLib.Variant('(s)', (interface.name,)), GLib.VariantType.new('(a{sv})'),
                                    Gio.DBusCallFlags.NONE,
                                    1000,
                                    None)[0]

                jsond[name][obj]['interfaces'][interface.name]['props'] = res

            except GLib.Error as e:
                pass

    except GLib.Error:
        pass
    

for name in names:
    for prefix in prefixes:
        if name.startswith(prefix):
            break
    else:
        # Prefix not found (loop finished)
        continue
    node_introspect(name, '/')

sys.stdout.write(json.dumps(jsond, indent=True))
