# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

import time
import math
import datetime
import shutil
import subprocess
import os
import os.path
import tempfile

# NOTE: This does not work with systemd because of issue #5629
def register_wakeup(reltime):

    tmpdir = tempfile.mkdtemp()
    fifo = os.path.join(tmpdir, 'wakeup-cmd-run')
    os.mkfifo(fifo)

#    dt = datetime.datetime.now()
#    dt = dt + datetime.timedelta(seconds=reltime + 1)
#    calendar = dt.strftime('%Y-%m-%d %H:%M:%S')
#    print('Sleeping until: ' + calendar)

    # NOTE: We use a calendar event just because 
    cmd = [
        'echo', 'systemd-run',
        '--description=Test wakeup in %is' % reltime,
        '--on-active=%i' % reltime,
        '--timer-property=WakeSystem=true',
        '--timer-property=AccuracySec=100ms',
        #'sh', '-c', 'echo 1 >%s' % fifo
        'true'
    ]

    subprocess.call(cmd)
    time.sleep(1)
    subprocess.call(['systemctl', 'list-timers'])

    handle = (tmpdir, fifo)
    return handle

def wait_wakeup(handle):
    f = open(handle[1], 'r', 0)
    f.read(1)

    shutil.rmtree(handle[0])


def suspend(sleeptime=None):
    # Sync disk to prevent sleep from kicking in later than intended
    subprocess.call(['sync'])

    start = time.time()

    # NOTE: Ideally this should register a wakeup with systemd and then
    #       suspend using that method. Unfortunately this is broken in
    #       older systemd releases (fixed in v234).
    #       https://github.com/systemd/systemd/issues/5629
    if sleeptime is None:
        subprocess.call(['systemctl', 'suspend'])
    else:
        subprocess.call(['rtcwake', '-s', str(sleeptime), '-m', 'mem'])
    slept = time.time() - start

    return slept

if __name__ == "__main__":
    #handle = register_wakeup(10)
    suspend(10)
    #wait_wakeup(handle)

