#!/usr/bin/env python2
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

from avocado import main

# Do not import Gtk or similar before importing the test class
from fed_laptoptest.test import SessionTest, batch
from fed_laptoptest.systemctl import suspend

class Suspend(SessionTest):
    """
    .. title:: Suspend and automatic resume

    This test suspends and lets it resume automatically after a fixed time.

    :avocado: enable
    :avocado: tags=standalone
    """

    SLEEPTIME = 20

    @batch
    def test(self):
        # XXX: The suspend call should throw an exception if suspend/resume
        #      fails in some way. This is not yet implemented.
        elapsed = suspend(self.SLEEPTIME)
        if elapsed < self.SLEEPTIME:
            self.fail("Computer did not go to sleep or did not sleep long enough!")
        if elapsed > self.SLEEPTIME + 5:
            self.fail("Computer slept for {:.1f}s seconds but should only sleep %d seconds (grace period: 5s)!".format(elapsed, self.SLEEPTIME))

if __name__ == "__main__":
    main()

