#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

from avocado import main
from avocado.core import exceptions
from fed_laptoptest import chameleon

# Do not import Gtk or similar before importing the test class
from fed_laptoptest.test import ChameleonTest, batch
from fed_laptoptest.edid import Edid
from fed_laptoptest import systemctl

import time

class SuspendPlug(ChameleonTest):
    """
    .. title:: Montior plugging while suspended

    This test plugs and unplugs monitors while the computer is suspended and
    then replugs/unplugs shortly after waking up again.

    :avocado: enable
    :avocado: tags=standalone,chameleon
    """

    # The following meanings:
    #  1. State before suspend
    #  2. State during suspend
    #  3. State right after wakeup
    PLUG_ORDER = [
        (False, True, True),
        (False, True, False),
        (False, False, True),

        (True, False, False),
        (True, False, True),
        (True, True, False),
    ]

    @batch
    def test(self):
        for port in self.chameleon.iter_displays():
            for order in self.PLUG_ORDER:

                self.log.info("Testing %i: plugged before/during/after suspend: %i/%i/%i"
                              % (port, order[0], order[2], order[1]))

                if order[0]:
                    self.chameleon.plug(port)
                    self.app_ctrl.monitor.wait_added()

                if order[0] != order[1]:
                    if order[1]:
                        self.chameleon.plug(port, delay=8)
                    else:
                        self.chameleon.unplug(port, delay=8)

                    # We do not wait in this case ...

                # Do the suspend action
                systemctl.suspend(10)

                if order[1] != order[2]:
                    if order[2]:
                        self.chameleon.plug(port)
                    else:
                        self.chameleon.unplug(port)

                if order[2]:
                    self.app_ctrl.monitor.wait_count(2)
                else:
                    self.app_ctrl.monitor.wait_no_external()

                self.chameleon.unplug(port)
                self.app_ctrl.monitor.wait_no_external()

                # XXX: It seems like suspend takes a long time if we are not
                # awake for a while. So sleep 10 seconds in between tests to
                # ensure the suspend action happens quickly.
                time.sleep(10)

if __name__ == "__main__":
    main()

